import type { ObjectFixedIntersectionToObject, CommonAdapterPool, CommonAdapterOptions, AdapterOperations, AdapterConnection, AdapterImplementation, StoreApi, PerformanceTracerLike, JsonMap, SearchCondition, UpdateCondition, ObjectFixedKeys, OmitObject } from '@resolve-js/readmodel-base';
import type PGLib from 'pg';
export * from '@resolve-js/readmodel-base';
export declare type LibDependencies = {
    Postgres: typeof PGLib.Client;
};
export declare type InlineLedgerRunQueryMethod = (querySQL: string, passthroughRuntimeErrors?: boolean) => Promise<Array<object>>;
export declare type FullJitterMethod = (retries: number) => Promise<void>;
export declare type MakeNestedPathMethod = (nestedPath: Array<string>) => string;
export declare type EscapeableMethod = (str: string) => string;
export declare type InlineLedgerForceStopMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type BuildUpsertDocumentMethod = (searchExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[3], updateExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[4]) => JsonMap;
export declare type RowLike = JsonMap;
export declare type MarshalledRowLike = RowLike & {
    marshalled: 'marshalled';
};
export declare type ConvertResultRowMethod = (inputRow: MarshalledRowLike, fieldList: Parameters<StoreApi<CommonAdapterPool>['find']>[4] | null) => RowLike;
export declare type SearchToWhereExpressionMethod = (expression: SearchCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod) => string;
export declare type UpdateToSetExpressionMethod = (expression: UpdateCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod) => string;
export interface PassthroughErrorInstance extends Error {
    name: string;
}
export declare type PassthroughErrorFactory = {
    new (): PassthroughErrorInstance;
} & {
    isPassthroughError: (error: Error & {
        code: string | number;
    }, includeRuntimeErrors: boolean) => boolean;
};
export declare type GenerateGuidMethod = (...args: any) => string;
export declare type DropReadModelMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type AdapterOptions = CommonAdapterOptions & {
    tablePrefix?: string;
    databaseName: string;
} & PGLib.ConnectionConfig;
export declare type InternalMethods = {
    inlineLedgerForceStop: InlineLedgerForceStopMethod;
    buildUpsertDocument: BuildUpsertDocumentMethod;
    convertResultRow: ConvertResultRowMethod;
    searchToWhereExpression: SearchToWhereExpressionMethod;
    updateToSetExpression: UpdateToSetExpressionMethod;
    PassthroughError: PassthroughErrorFactory;
    generateGuid: GenerateGuidMethod;
    dropReadModel: DropReadModelMethod;
    escapeId: EscapeableMethod;
    escapeStr: EscapeableMethod;
};
export declare type ArrayOrSingleOrNull<T> = Array<T> | T | null;
export declare type UpdateFieldDescriptor = {
    [K in ObjectFixedKeys<ObjectFixedIntersectionToObject<UpdateCondition>>]: ObjectFixedIntersectionToObject<UpdateCondition>[K][string] | null;
} & {
    key: string;
    nestedKey: string[];
    baseName: string;
    selectedOperation: ObjectFixedKeys<UpdateCondition> | null;
    children: Map<string, UpdateFieldDescriptor>;
    operations?: ArrayOrSingleOrNull<{
        operationName: ObjectFixedKeys<UpdateCondition> | null;
        fieldValue: UpdateFieldDescriptor[ObjectFixedKeys<UpdateCondition>];
        nestedPath?: string[];
    }>;
};
export declare type AdapterPool = CommonAdapterPool & {
    inlineLedgerRunQuery: InlineLedgerRunQueryMethod;
    performanceTracer: PerformanceTracerLike;
    tablePrefix: string;
    schemaName: string;
    makeNestedPath: MakeNestedPathMethod;
    activePassthrough: boolean;
    connection: InstanceType<LibDependencies['Postgres']>;
} & {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterOperations<AdapterPool>[K];
} & {
    [K in keyof StoreApi<CommonAdapterPool>]: StoreApi<AdapterPool>[K];
} & InternalMethods;
export declare type CurrentAdapterConnection = AdapterConnection<AdapterPool, OmitObject<AdapterOptions, CommonAdapterPool>>;
export declare type ExternalMethods = {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterPool[K];
};
export declare type CurrentStoreApi = {
    [K in keyof StoreApi<CommonAdapterPool>]: AdapterPool[K];
};
export declare type ConnectionDependencies = LibDependencies & InternalMethods & ExternalMethods & CurrentStoreApi;
export declare type CurrentConnectMethod = (imports: ConnectionDependencies, ...args: Parameters<CurrentAdapterConnection['connect']>) => ReturnType<CurrentAdapterConnection['connect']>;
export declare type CurrentDisconnectMethod = CurrentAdapterConnection['disconnect'];
export declare type CurrentAdapterImplementation = AdapterImplementation<AdapterPool, AdapterOptions>;
